//
//  RecorderViewController.m
//  Week8_AudioVideo
//
//  Created by Jason Fieldman on 3/10/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "RecorderViewController.h"
#import <AudioToolbox/AudioToolbox.h>

#define kMeterXOffset   40
#define kMeterWidth     240
#define kMeterYOffset   200
#define kMeterHeight    40



@implementation RecorderViewController

- (id) init {
	if (self = [super init]) {
		self.title = @"Recorder";
		self.view = [[[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 480)] autorelease];
		
		/* Create recording path */
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
		NSString *cacheDirectory = [paths objectAtIndex:0];
		NSString *filePath = [NSString stringWithFormat:@"%@/recording.wav", cacheDirectory];
		fileURL = [[NSURL alloc] initFileURLWithPath:filePath];
		
		/* Create buttons */
		recordButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
		recordButton.frame = CGRectMake(10, 40, 145, 60);
		[recordButton setTitle:@"Record" forState:UIControlStateNormal];
		[recordButton addTarget:self action:@selector(handleRecord:) forControlEvents:UIControlEventTouchUpInside];
		[self.view addSubview:recordButton];
		
		playbackButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
		playbackButton.frame = CGRectMake(165, 40, 145, 60);
		[playbackButton setTitle:@"Play Back" forState:UIControlStateNormal];
		[playbackButton addTarget:self action:@selector(handlePlayback:) forControlEvents:UIControlEventTouchUpInside];
		[self.view addSubview:playbackButton];
		
		/* Status bar */
		statusLabel = [[[UILabel alloc] initWithFrame:CGRectMake(0, 120, 320, 24)] autorelease];
		statusLabel.textAlignment = UITextAlignmentCenter;
		[self.view addSubview:statusLabel];
		
		/* Sound meter */
		meterBackground = [[[UIView alloc] initWithFrame:CGRectMake(kMeterXOffset, kMeterYOffset, kMeterWidth, kMeterHeight)] autorelease];
		meterBackground.backgroundColor = [UIColor darkGrayColor];
		[self.view addSubview:meterBackground];
		
		meterForeground = [[[UIView alloc] initWithFrame:CGRectZero] autorelease];
		meterForeground.backgroundColor = [UIColor lightGrayColor];
		[self.view addSubview:meterForeground];
	}
	return self;
}


- (void) haltAudio {
	if (player) {
		[player stop];
		[player release];
		player = nil;
		[playbackButton setTitle:@"Play Back" forState:UIControlStateNormal];
	}
	
	if (recorder) {
		[recorder stop];
		[recorder release];
		recorder = nil;
		[recordButton setTitle:@"Record" forState:UIControlStateNormal];
	}
}


- (void)dealloc {
	[self haltAudio];
	[fileURL release];
    [super dealloc];
}


- (void) setMeterToLevel:(float)level {
	NSLog(@"setMeter: %f", level);
	
	if (level < 0) level = 0;
	if (level > 1) level = 1;
	int newMeterWidth = (kMeterWidth - 2) * level;
	
	meterForeground.frame = CGRectMake(kMeterXOffset + 1, kMeterYOffset + 1, newMeterWidth, kMeterHeight - 2);
}


- (void) updateStatus {
	if (recorder.recording) {
		statusLabel.text = [NSString stringWithFormat:@"Recording: %.2lfs", recorder.currentTime];
		[recorder updateMeters];		
		[self setMeterToLevel:(([recorder peakPowerForChannel:0]+80)/80.0)];
	} else if (player.playing) {
		statusLabel.text = [NSString stringWithFormat:@"Playing: %.2lfs of %.2lf", player.currentTime, player.duration];
		[player updateMeters];		
		[self setMeterToLevel:(([player peakPowerForChannel:0]+80)/80.0)];
	}
}


- (void) handleUpdateTimer:(NSTimer*)timer {
	/* First off, kill the timer if we're not recording or playing */
	if (!recorder.recording && !player.playing) {
		[timer invalidate];
		statusLabel.text = @"Done";
		return;
	}
	
	[self updateStatus];
}


- (void) handleRecord:(id)sender {
	/* Were we already recording? If so, just stop */
	if (recorder) {
		/* We needed to check the recorder variable before halting audio */
		[self haltAudio];
		return;
	}
	
	/* Turn off any existing audio */
	[self haltAudio];
	
	/* Record settings */
	NSMutableDictionary *settings = [NSMutableDictionary dictionary];
	[settings setObject:[NSNumber numberWithInt:kAudioFormatLinearPCM] forKey:AVFormatIDKey];
	[settings setObject:[NSNumber numberWithFloat:44100.0] forKey:AVSampleRateKey];

	/* Create recorder and start it up */
	recorder = [[AVAudioRecorder alloc] initWithURL:fileURL settings:settings error:nil];
	recorder.meteringEnabled = YES;
	[recorder record];
	
	/* Change title of the record button to stop */
	[recordButton setTitle:@"Stop Recording" forState:UIControlStateNormal];
	
	/* Start the status timer */
	[NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(handleUpdateTimer:) userInfo:nil repeats:YES];
}


- (void) handlePlayback:(id)sender {
	/* Were we already playing? If so, just stop */
	if (player) {
		/* We needed to check the player variable before halting audio */
		[self haltAudio];
		return;
	}
	
	/* Turn off any existing audio */
	[self haltAudio];
	
	/* Create player and start it up */
	player = [[AVAudioPlayer alloc] initWithContentsOfURL:fileURL error:nil];
	player.meteringEnabled = YES;
	[player play];
	
	/* Change title of the record button to stop */
	[playbackButton setTitle:@"Stop Playing" forState:UIControlStateNormal];
	
	/* Start the status timer */
	[NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(handleUpdateTimer:) userInfo:nil repeats:YES];
}


@end
