//
//  SoundboardViewController.m
//  Week8_AudioVideo
//
//  Created by Jason Fieldman on 3/10/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "SoundboardViewController.h"

#define kButtonXOffset  10
#define kButtonXSpacing 80
#define kButtonWidth    60
#define kButtonYOffset  40
#define kButtonHeight   60

#define kActXOffset  30
#define kActXSpacing 80
#define kActWidth    20
#define kActYOffset  120
#define kActHeight   20


@implementation SoundboardViewController


- (id) init {
	if (self = [super init]) {
		self.title = @"Soundboard";
		self.view = [[[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 480)] autorelease];
		
		for (int i = 0; i < NUM_SOUNDS; i++) {

			/* Initialize the audio player for resource sound_X.aiff */
			NSString *soundFileName = [NSString stringWithFormat:@"sound_%d", i+1];
			NSString *soundFilePath = [[NSBundle mainBundle] pathForResource:soundFileName ofType:@"aiff" inDirectory:@"/"];
			NSURL *soundURL = [NSURL fileURLWithPath:soundFilePath];
			audioPlayers[i] = [[AVAudioPlayer alloc] initWithContentsOfURL:soundURL error:nil];
			audioPlayers[i].delegate = self;
			
			/* Initialize and add the buttons */
			buttons[i] = [UIButton buttonWithType:UIButtonTypeRoundedRect];
			buttons[i].frame = CGRectMake(kButtonXOffset + (i * kButtonXSpacing), kButtonYOffset, kButtonWidth, kButtonHeight);
			[buttons[i] setTitle:[NSString stringWithFormat:@"Play %d", i+1] forState:UIControlStateNormal];
			[buttons[i] addTarget:self action:@selector(handleButtonPress:) forControlEvents:UIControlEventTouchUpInside];
			[self.view addSubview:buttons[i]];
								
			/* Initialize and add the activity indicators */
			activityIndicators[i] = [[[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray] autorelease];
			activityIndicators[i].frame = CGRectMake(kActXOffset + (i * kActXSpacing), kActYOffset, kActWidth, kActHeight);
			activityIndicators[i].hidesWhenStopped = YES;
			[activityIndicators[i] stopAnimating];
			[self.view addSubview:activityIndicators[i]];
		}
		
		
	}
	return self;
}


- (void)dealloc {
	for (int i = 0; i < NUM_SOUNDS; i++) {
		[audioPlayers[i] release];
	}
	
    [super dealloc];
}


- (void) handleButtonPress:(id)sender {
	/* identify the sender */
	int index = -1;
	for (int i = 0; i < NUM_SOUNDS; i++) {
		if (sender == buttons[i]) {
			index = i;
			break;
		}
	}
	
	/* No valid sender? */
	if (index == -1) return;
	
	/* Reset the audio player to the beginning */
	[audioPlayers[index] stop];
	audioPlayers[index].currentTime = 0;
	[audioPlayers[index] play];
	
	/* Show the playing indicator */
	[activityIndicators[index] startAnimating];
	
}


- (void)audioPlayerDidFinishPlaying:(AVAudioPlayer *)player successfully:(BOOL)flag {
	/* identify the sender */
	int index = -1;
	for (int i = 0; i < NUM_SOUNDS; i++) {
		if (player == audioPlayers[i]) {
			index = i;
			break;
		}
	}
	
	/* Invalid audio player? */
	if (index == -1) return;
	
	/* Stop the activity indicator */
	[activityIndicators[index] stopAnimating];
}


@end
