//
//  StreamingMusicViewController.m
//  Week8_AudioVideo
//
//  Created by Jason Fieldman on 3/10/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "StreamingMusicViewController.h"


@implementation StreamingMusicViewController

- (id) init {
	if (self = [super init]) {
		self.title = @"Streaming";
		self.view = [[[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 480)] autorelease];
		
		/* Initialize views */
		remoteURLField = [[[UITextField alloc] initWithFrame:CGRectMake(20, 40, 200, 24)] autorelease];
		remoteURLField.backgroundColor = [UIColor lightGrayColor];
		remoteURLField.text = @"http://www.publicbroadcasting.net/wxxi/.jukebox/media/wxxi/889272/mp3/1370connection/podcast/10263/889272.mp3";
		[self.view addSubview:remoteURLField];
		
		controlButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
		controlButton.frame = CGRectMake(240, 40, 60, 24);
		[controlButton setTitle:@"Play" forState:UIControlStateNormal];
		[controlButton addTarget:self action:@selector(handleControlButton:) forControlEvents:UIControlEventTouchUpInside];
		[self.view addSubview:controlButton];
		
		statusLabel = [[[UILabel alloc] initWithFrame:CGRectMake(0, 80, 320, 24)] autorelease];
		statusLabel.textAlignment = UITextAlignmentCenter;
		[self.view addSubview:statusLabel];
	}
	return self;
}


- (void)dealloc {
    [super dealloc];
}


- (void) destroyStreamer {
	if (streamer) {
		[[NSNotificationCenter defaultCenter]
		 removeObserver:self
		 name:ASStatusChangedNotification
		 object:streamer];
		
		[streamer stop];
		[streamer release];
		streamer = nil;
	}
}


- (void) createStreamer {
	if (streamer) {
		return;
	}
	
	[self destroyStreamer];
	
	NSString *escapedValue = [(NSString *)CFURLCreateStringByAddingPercentEscapes(
														 nil,
														 (CFStringRef)remoteURLField.text,
														 NULL,
														 NULL,
														 kCFStringEncodingUTF8)
	 autorelease];
	
	NSURL *url = [NSURL URLWithString:escapedValue];
	streamer = [[AudioStreamer alloc] initWithURL:url];
	
	[[NSNotificationCenter defaultCenter]
	 addObserver:self
	 selector:@selector(playbackStateChanged:)
	 name:ASStatusChangedNotification
	 object:streamer];
}


//
- (void)playbackStateChanged:(NSNotification *)aNotification
{
	if ([streamer isWaiting]) {
		statusLabel.text = @"Buffering...";
	} else if ([streamer isPlaying]) {
		statusLabel.text = @"Playing!";
	} else if ([streamer isIdle]) {
		statusLabel.text = @"Stopped";
		[self destroyStreamer];
		[controlButton setTitle:@"Play" forState:UIControlStateNormal];
	}
}



- (void) handleControlButton:(id)sender {
	/* Close keyboard */
	[remoteURLField resignFirstResponder];
	
	if (!streamer && [remoteURLField.text length]) {
		/* We need to start the streamer */
		[self createStreamer];
		[streamer start];
		[controlButton setTitle:@"Stop" forState:UIControlStateNormal];
		return;
	}
	
	if (streamer) {
		/* We need to stop the streamer */
		[self destroyStreamer];
		statusLabel.text = @"Stopped";
		[controlButton setTitle:@"Play" forState:UIControlStateNormal];
		return;
	}
}


@end
