//
//  iPodViewController.m
//  Week8_AudioVideo
//
//  Created by Jason Fieldman on 3/10/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "iPodViewController.h"

@interface iPodViewController (priv)
- (void) updateInfoForCurrentTrack;
- (void) updatePlayButtonForCurrentStatus;
@end



@implementation iPodViewController

- (id) init {
	if (self = [super init]) {
		self.title = @"iPod Player";
		self.view = [[[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 480)] autorelease];
		
		#if TARGET_IPHONE_SIMULATOR
		musicController = nil;
		#else
		musicController = [MPMusicPlayerController iPodMusicPlayer];
		#endif
		
		
		/* Progress info */
		progressView = [[[UIProgressView alloc] initWithFrame:CGRectMake(80, 30, 160, 30)] autorelease];
		[self.view addSubview:progressView];
		
		curtimeLabel = [[[UILabel alloc] initWithFrame:CGRectMake(0, 19, 70, 30)] autorelease];
		curtimeLabel.font = [UIFont systemFontOfSize:14];
		curtimeLabel.textAlignment = UITextAlignmentRight;
		curtimeLabel.text = @"0:00";
		[self.view addSubview:curtimeLabel];
		
		totaltimeLabel = [[[UILabel alloc] initWithFrame:CGRectMake(250, 19, 70, 30)] autorelease];
		totaltimeLabel.font = [UIFont systemFontOfSize:14];
		totaltimeLabel.textAlignment = UITextAlignmentLeft;
		totaltimeLabel.text = @"0:00";
		[self.view addSubview:totaltimeLabel];
		
		/* Track labels */
		titleLabel = [[[UILabel alloc] initWithFrame:CGRectMake(0, 75, 320, 24)] autorelease];
		titleLabel.font = [UIFont boldSystemFontOfSize:20];
		titleLabel.textAlignment = UITextAlignmentCenter;
		titleLabel.text = @"----";
		[self.view addSubview:titleLabel];
		
		artistLabel = [[[UILabel alloc] initWithFrame:CGRectMake(0, 50, 320, 24)] autorelease];
		artistLabel.font = [UIFont systemFontOfSize:16];
		artistLabel.textAlignment = UITextAlignmentCenter;
		artistLabel.text = @"----";
		[self.view addSubview:artistLabel];
		
		albumLabel = [[[UILabel alloc] initWithFrame:CGRectMake(0, 100, 320, 24)] autorelease];
		albumLabel.font = [UIFont systemFontOfSize:16];
		albumLabel.textAlignment = UITextAlignmentCenter;
		albumLabel.text = @"----";
		[self.view addSubview:albumLabel];
		
		/* Track art */
		trackartImage = [[[UIImageView alloc] initWithFrame:CGRectMake(96, 140, 128, 128)] autorelease];
		trackartImage.backgroundColor = [UIColor clearColor];
		[self.view addSubview:trackartImage];
		
		/* Control buttons */
		prevButton = [UIButton buttonWithType:UIButtonTypeCustom];
		prevButton.frame = CGRectMake(20, 290, 64, 64);		
		[prevButton setImage:[UIImage imageNamed:@"prev.png"] forState:UIControlStateNormal];
		[prevButton addTarget:self action:@selector(handlePrev:) forControlEvents:UIControlEventTouchUpInside];		
		[self.view addSubview:prevButton];
		
		playButton = [UIButton buttonWithType:UIButtonTypeCustom];
		playButton.frame = CGRectMake(90, 290, 64, 64);		
		[playButton setImage:[UIImage imageNamed:@"play.png"] forState:UIControlStateNormal];
		[playButton addTarget:self action:@selector(handlePlay:) forControlEvents:UIControlEventTouchUpInside];		
		[self.view addSubview:playButton];
		
		nextButton = [UIButton buttonWithType:UIButtonTypeCustom];
		nextButton.frame = CGRectMake(160, 290, 64, 64);		
		[nextButton setImage:[UIImage imageNamed:@"next.png"] forState:UIControlStateNormal];
		[nextButton addTarget:self action:@selector(handleNext:) forControlEvents:UIControlEventTouchUpInside];		
		[self.view addSubview:nextButton];
		
		listButton = [UIButton buttonWithType:UIButtonTypeCustom];
		listButton.frame = CGRectMake(230, 290, 64, 64);		
		[listButton setImage:[UIImage imageNamed:@"playlist.png"] forState:UIControlStateNormal];
		[listButton addTarget:self action:@selector(handleList:) forControlEvents:UIControlEventTouchUpInside];		
		[self.view addSubview:listButton];
				
		/* Update UI */
		[self updateInfoForCurrentTrack];
		[self updatePlayButtonForCurrentStatus];
		
		/* Register notifications */
		NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
		
		[notificationCenter addObserver: self
							   selector: @selector (handleNowPlayingItemChanged:)
								   name: MPMusicPlayerControllerNowPlayingItemDidChangeNotification
								 object: musicController];
		
		[notificationCenter addObserver: self
							   selector: @selector (handlePlaybackStateChanged:)
								   name: MPMusicPlayerControllerPlaybackStateDidChangeNotification
								 object: musicController];
		
		[musicController beginGeneratingPlaybackNotifications];
		
		/* Start timer that updates progress meter */
		[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(handleUpdateTimer:) userInfo:nil repeats:YES];
	}
	return self;
}


- (void)dealloc {
    [super dealloc];
}


- (void) updateInfoForCurrentTrack {
	BOOL hasItem = (musicController.nowPlayingItem != nil);
	
	/* Affect hide state of the progress views */
	curtimeLabel.hidden = !hasItem;
	totaltimeLabel.hidden = !hasItem;
	progressView.hidden = !hasItem;
	trackartImage.hidden = !hasItem;
	
	/* Take care of the case where nothing is playing */
	if (!hasItem) {
		titleLabel.text = @"No Media Item Selected";
		albumLabel.text = nil;
		artistLabel.text = nil;
		trackartImage.image = nil;
		return;
	}
	
	/* Otherwise we have an item */
	MPMediaItem *item = musicController.nowPlayingItem;
	
	/* Update progress */	
	curtimeLabel.text   = [NSString stringWithFormat:@"%02d:%02d", (int)musicController.currentPlaybackTime / 60, (int)musicController.currentPlaybackTime % 60];
	
	NSNumber *dur = [item valueForProperty:MPMediaItemPropertyPlaybackDuration];
	int dur_secs = [dur intValue];
	totaltimeLabel.text = [NSString stringWithFormat:@"%02d:%02d", dur_secs / 60, dur_secs % 60];
		
	float progress = musicController.currentPlaybackTime / dur_secs;
	progressView.progress = progress;
	
	/* Update track info */
	artistLabel.text = [item valueForProperty:MPMediaItemPropertyArtist];
	titleLabel.text  = [item valueForProperty:MPMediaItemPropertyTitle];
	albumLabel.text  = [item valueForProperty:MPMediaItemPropertyAlbumTitle];
	
	/* Update artwork */
	MPMediaItemArtwork *artwork = [item valueForProperty:MPMediaItemPropertyArtwork];
	UIImage *artImage = [artwork imageWithSize:CGSizeMake(128, 128)];
	trackartImage.image = artImage;
}


- (void) handleUpdateTimer:(NSTimer*)timer {
	if (musicController.playbackState == MPMusicPlaybackStatePlaying) {
		[self updateInfoForCurrentTrack];
	}
}


- (void) updatePlayButtonForCurrentStatus {
	[playButton setImage:(musicController.playbackState == MPMusicPlaybackStatePlaying) ? [UIImage imageNamed:@"pause.png"] : [UIImage imageNamed:@"play.png"] forState:UIControlStateNormal];
}


- (void) handleNowPlayingItemChanged: (id) notification {
	/* Update the track info if we naturally went to the next track */
	[self updateInfoForCurrentTrack];
}

- (void) handlePlaybackStateChanged: (id) notification {
	/* Update the play button if we naturally stopped playing */
	[self updatePlayButtonForCurrentStatus];
}


- (void) handlePlay:(id)sender {
	if (musicController.playbackState == MPMusicPlaybackStatePlaying) {
		[musicController pause];
	} else if (musicController.nowPlayingItem) {
		[musicController play];
	}
	[self updatePlayButtonForCurrentStatus];
}

- (void) handlePrev:(id)sender {
	if (musicController.currentPlaybackTime < 2) {
		[musicController skipToPreviousItem];
	} else {
		[musicController skipToBeginning];
	}
	[self updateInfoForCurrentTrack];
	[self updatePlayButtonForCurrentStatus];
}

- (void) handleNext:(id)sender {	
	[musicController skipToNextItem];
	[self updateInfoForCurrentTrack];
	[self updatePlayButtonForCurrentStatus];
}

- (void) handleList:(id)sender {
	MPMediaPickerController *picker = [[[MPMediaPickerController alloc] initWithMediaTypes: MPMediaTypeAny] autorelease];
	
	picker.delegate						= self;
	picker.allowsPickingMultipleItems	= YES;
	picker.prompt						= @"Choose Media To Play";
		
	[self presentModalViewController:picker animated:YES];
}


#pragma mark MPMediaPickerControllerDelegate methods

- (void)mediaPicker: (MPMediaPickerController *)mediaPicker didPickMediaItems:(MPMediaItemCollection *)mediaItemCollection {	
	if (!mediaItemCollection) return;

	/* Update music player */
	[musicController setQueueWithItemCollection:mediaItemCollection];
	
	/* Force play */
	[musicController play];
	
	/* Update info */
	[self updateInfoForCurrentTrack];
	[self updatePlayButtonForCurrentStatus];
	
	/* Dismiss picker */
	[self dismissModalViewControllerAnimated: YES];
}

- (void)mediaPickerDidCancel:(MPMediaPickerController *)mediaPicker {
	[self dismissModalViewControllerAnimated:YES];
}


@end
